#version 100

uniform sampler2D TextureYUV;
uniform sampler2D TextureMask;
varying lowp vec2 TexCoordInterpolated;
uniform lowp vec2 PixelDelta;
uniform lowp float ThresholdY;

#define FILTER_SIZE 17
#define HALF_FILTER_SIZE int(FILTER_SIZE / 2)
uniform lowp float FilterCoeffs[FILTER_SIZE];

void main(void) {
    
    lowp vec4 sum = vec4(0.0);
    
    for (int  i = 0; i < FILTER_SIZE; i++) {
        
        // Get YUV sample
        lowp vec2 texCoord = TexCoordInterpolated + vec2(float(i - HALF_FILTER_SIZE) * PixelDelta.x, 0.0);
        lowp vec4 yuv = texture2D(TextureYUV, texCoord);
        
        // Bias Y value
        lowp float B = 1.0 - ThresholdY;
        lowp float W = clamp(yuv.x - ThresholdY, 0.0, 1.0);
        lowp float yBiased = clamp(yuv.x + B, 0.0, 1.0);
        
        // Blend original Y with biased Y, according to the mask
        lowp vec4 mask = texture2D(TextureMask, texCoord);
        yuv.r = mix(yBiased, yuv.r, mask.r);
        
        // Accumulate
        sum += yuv * vec4(FilterCoeffs[i]);
    }

    gl_FragColor = vec4(sum.rgb, 1.0);
}