#version 100

uniform sampler2D TextureYUV;
varying lowp vec2 TexCoordInterpolated;
uniform lowp vec2 PixelDelta;
uniform lowp float ThresholdY;
uniform lowp float ThresholdUVSquared;

#define FILTER_SIZE 17
#define HALF_FILTER_SIZE int(FILTER_SIZE / 2)
uniform lowp float FilterCoeffs[FILTER_SIZE];

lowp float yuv_to_mask(lowp vec4 yuv) {

    // (y < Ty) OR (u^2 + v^2 > Tuv^2)
    //
    // v
    //
    // (y < Ty) OR (Tuv^2 < u^2 + v^2)
    //
    // v
    //
    // any( lessThan( vec2(y, Tuv^2), vec2(Ty, u^2, v^2) ) )
    
    lowp vec2 uvOffset = yuv.yz - vec2(0.5);
    lowp vec2 uv2 = uvOffset * uvOffset;
    lowp vec2 a = vec2(yuv.x, ThresholdUVSquared);
    lowp vec2 b = vec2(ThresholdY, uv2.x + uv2.y);
    bvec2 m = lessThan(a, b);
    return float(any(m));
}

void main(void) {
    
    lowp float sum = 0.0;
    for (int  i = 0; i < FILTER_SIZE; i++) {
        
        lowp vec2 texCoord = TexCoordInterpolated + vec2(float(i - HALF_FILTER_SIZE) * PixelDelta.x, 0.0);
        lowp vec4 yuv = texture2D(TextureYUV, texCoord);
        sum += yuv_to_mask(yuv) * FilterCoeffs[i];
    }

    gl_FragColor = vec4(vec3(sum), 1.0);
}